/*
	Copyright (C) 1995-2000 AROS - The Amiga Research OS
	$Id: lock.c,v 1.1.1.1 2005/03/15 16:01:00 laire Exp $
 
	Desc: Lock - Set the write protect status of a device.
	Lang: english
*/

/**************************************************************************
 
	NAME
	Lock
 
	FORMAT
	Lock <drive> [ON|OFF] [<passkey>]
 
	SYNOPSIS
	DRIVE/A,ON/S,OFF/S,PASSKEY
 
	LOCATION
	Workbench:c
 
	FUNCTION
	Lock will cause the specified device or partition to be made write-
	protected or write-enabled. This write protection is a soft write
	protection which is handled by the volume filesystem. Hence the
	protection will be reset (to writable) on the next system reboot.
 
	It is possible to specify an optional passkey which can be used to
	password protect the locking. The same passkey that is used to lock
	the volume must be used to unlock the volume. The passkey may be
	any number of characters in length.
 
	The volume given MUST be the device or root volume name, not an
	assign.
 
	EXAMPLE
 
	1.SYS:> Lock Work:
 
	    This will lock the volume called Work: without a passkey.
 
	1.SYS:> Lock Work:
	1.SYS:> MakeDir Work:SomeDir
	Can't create directory Work:Test
	MakeDir: Disk is write-protected
 
	    The volume Work: is locked, so it is impossible to create a
	    directory.
 
	1.SYS:> Lock Work: OFF
 
	    This will unlock the volume work.
 
	1.SYS:> Lock Work: MyPassword
 
	    This will lock Work: with the passkey "MyPassword"
 
**************************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <dos/dosextens.h>
#include <dos/exall.h>
#include <dos/rdargs.h>
#include <utility/tagitem.h>

#include <proto/exec.h>
#include <proto/dos.h>

#define ARG_TEMPLATE	"DRIVE/A,ON/S,OFF/S,PASSKEY"
enum
{
	ARG_DRIVE,
	ARG_ON,
	ARG_OFF,
	ARG_PASSKEY,
	NUMARGS
};

#include "lock_version.h"

static const char version[] = VERSTAG;
static const char exthelp[] =
	"Lock : Set the write-protect state of a volume\n"
	"\tDRIVE/A   The name of the volume to lock\n"
	"\tON/S      Turn write protect on\n"
	"\tOFF/S     Turn write protect off\n"
	"\tPASSKEY   The password string for the lock (optional)\n";


int main(void)
{
	struct ExecBase *SysBase;
	struct DosLibrary	*DOSBase;
	struct RDArgs *rd, *rda = NULL;
	IPTR args[NUMARGS] = { NULL, FALSE, FALSE, NULL };
	struct DevProc *dp;
	int error = 0;
	int result = RETURN_FAIL;

	SysBase = *((struct ExecBase**) 4);
	DOSBase = (struct DosLibrary *)OpenLibrary("dos.library",37);
	if (DOSBase)
	{
		rda = AllocDosObject(DOS_RDARGS, NULL);
		if (rda)
		{
			rda->RDA_ExtHelp = (STRPTR)exthelp;
			rd = ReadArgs(ARG_TEMPLATE, args, rda);
			if (rd)
			{
				/* Firstly, find out the volume */
				dp = GetDeviceProc((STRPTR)args[ARG_DRIVE], NULL);

				if (dp == NULL ||
				    dp->dvp_DevNode->dol_Type < DLT_DEVICE ||
				    dp->dvp_DevNode->dol_Type > DLT_VOLUME)
				{
					error = dp ? ERROR_OBJECT_WRONG_TYPE : IoErr();
				}
				else
				{
					ULONG key = 0;

					/*
					 *  It is a real volume, not an assign...
					 *  We send the device a I/O request.
					 */

					/* calculate key, if needed */
					if (args[ARG_PASSKEY])
					{
						CONST_STRPTR s;
						UBYTE c;

						for (s = (CONST_STRPTR) args[ARG_PASSKEY]; (c = *s); s++)
						{
							key = key * 10 + c;
						}
					}

					if (args[ARG_ON])
					{
						error = DoPkt2(dp->dvp_Port, ACTION_WRITE_PROTECT, DOSTRUE, key);
						if (!error)
						{
							error = IoErr();
							result = RETURN_FAIL;
							Printf("Attempt to lock drive %s failed\n", args[ARG_DRIVE]);
						}
						else
						{
							error = 0;
							Printf("%s locked\n", args[ARG_DRIVE]);
						}
					}
					else if (args[ARG_OFF])
					{
						error = DoPkt2(dp->dvp_Port, ACTION_WRITE_PROTECT, DOSFALSE, key);
						if (!error)
						{
							error = IoErr();
							result = RETURN_FAIL;
							Printf("Attempt to unlock drive %s failed\n", args[ARG_DRIVE]);
						}
						else
						{
							error = 0;
							Printf("%s unlocked\n", args[ARG_DRIVE]);
						}
					}
				}

				if (dp)
				{
					FreeDeviceProc(dp);
				}

				FreeArgs(rd);

				if (!error)
				{
					result = RETURN_OK;
				}

			} /* if (rd) */
			else
			{
				error = IoErr();
			}

			FreeDosObject(DOS_RDARGS, rda);

		} /* if (rda) */
		else
		{
			error = ERROR_NO_FREE_STORE;
		}

		if (error)
		{
			PrintFault(error, "Lock");
			SetIoErr(error);
			result = RETURN_FAIL;
		}

		CloseLibrary((struct Library *) DOSBase);
	}

	return result;
}
